#if !defined(CGAME_H_INCLUDED)
#define CGAME_H_INCLUDED

#include "stdafx.h"
#include "mathlib.h"
#include "Packet.h"

// playerState_t is the information needed by both the client and server
// to predict player motion and actions
// nothing outside of pmove should modify these, or some degree of prediction error
// will occur

// you can't add anything to this without modifying the code in msg.c

// playerState_t is a full superset of entityState_t as it is used by players,
// so if a playerState_t is transmitted, the entityState_t can be fully derived
// from it.
typedef struct playerState_s {
	int			commandTime;	// cmd->serverTime of last executed command
	int			pm_type;		// 
	int			bobCycle;		// for view bobbing and footstep generation
	int			pm_flags;		// ducked, jump_held, etc
	int			pm_time;

	vec3_t		origin;
	vec3_t		velocity;
	int			weaponTime;
	int			gravity;
	int			speed;
	int			delta_angles[3];	// add to command angles to get view direction
									// changed by spawns, rotating objects, and teleporters

	int			groundEntityNum;// ENTITYNUM_NONE = in air

	int			legsTimer;		// don't change low priority animations until this runs out
	int			legsAnim;		// mask off ANIM_TOGGLEBIT

	int			torsoTimer;		// don't change low priority animations until this runs out
	int			torsoAnim;		// mask off ANIM_TOGGLEBIT

	int			movementDir;	// a number 0 to 7 that represents the reletive angle
								// of movement to the view angle (axial and diagonals)
								// when at rest, the value will remain unchanged
								// used to twist the legs during strafing

	vec3_t		grapplePoint;	// location of grapple to pull towards if PMF_GRAPPLE_PULL

	int			eFlags;			// copied to entityState_t->eFlags

	int			eventSequence;	// pmove generated events
	int			events[MAX_PS_EVENTS];
	int			eventParms[MAX_PS_EVENTS];

	int			externalEvent;	// events set on player from another source
	int			externalEventParm;
	int			externalEventTime;

	int			clientNum;		// ranges from 0 to MAX_CLIENTS-1
	int			weapon;			// copied to entityState_t->weapon
	int			weaponstate;

	vec3_t		viewangles;		// for fixed views
	int			viewheight;

	// damage feedback
	int			damageEvent;	// when it changes, latch the other parms
	int			damageYaw;
	int			damagePitch;
	int			damageCount;

	int			stats[MAX_STATS];
	int			persistant[MAX_PERSISTANT];	// stats that aren't cleared on death
	int			powerups[MAX_POWERUPS];	// level.time that the powerup runs out
	int			ammo[MAX_WEAPONS];

	// not communicated over the net at all
	int			ping;			// server to game info for scoreboard
} playerState_t;

typedef enum {
	TR_STATIONARY,
	TR_INTERPOLATE,				// non-parametric, but interpolate between snapshots
	TR_LINEAR,
	TR_LINEAR_STOP,
	TR_SINE,					// value = base + sin( time / duration ) * delta
	TR_GRAVITY
} trType_t;

typedef struct {
	//trType_t	trType;
	int		trType;
	int		trTime;
	int		trDuration;			// if non 0, trTime + trDuration = stop time
	vec3_t	trBase;
	vec3_t	trDelta;			// velocity, etc
} trajectory_t;

// entityState_t is the information conveyed from the server
// in an update message about entities that the client will
// need to render in some way
// Different eTypes may use the information in different ways
// The messages are delta compressed, so it doesn't really matter if
// the structure size is fairly large

typedef struct entityState_s {

	int		number;			// entity index
	int		eType;			// entityType_t
	int		eFlags;

	trajectory_t	pos;	// for calculating position
	trajectory_t	apos;	// for calculating angles

	int		time;
	int		time2;

	vec3_t	origin;
	vec3_t	origin2;

	vec3_t	angles;
	vec3_t	angles2;

	int		otherEntityNum;	// shotgun sources, etc
	int		otherEntityNum2;

	int		groundEntityNum;	// -1 = in air

	int		constantLight;	// r + (g<<8) + (b<<16) + (intensity<<24)
	int		loopSound;		// constantly loop this sound

	int		modelindex;
	int		modelindex2;
	int		clientNum;		// 0 to (MAX_CLIENTS - 1), for players and corpses
	int		frame;

	int		solid;			// for client side prediction, trap_linkentity sets this properly

	int		event;			// impulse events -- muzzle flashes, footsteps, etc
	int		eventParm;

	// for players
	int		powerups;		// bit flags
	int		weapon;			// determines weapon and flash model, etc
	int		legsAnim;		// mask off ANIM_TOGGLEBIT
	int		torsoAnim;		// mask off ANIM_TOGGLEBIT

} entityState_t;

// daten eines Spielers
typedef struct clientInfo_s
{
	char			name[MAX_QPATH];	// Spielername ohne colorzeichen
	BYTE			team;				// sein Team
	int				botSkill;		// 0 = not bot, 1-5 = bot
	int				color;			// effektfarbe
	int				score;			// updated by score servercmds
	int				location;		// location index for team mode
	int				health;			// you only get this info about your teammates
	int				armor;
	int				curWeapon;
	int				gameEntered;	// wann der Spieler in das spiel kam
	int				gameLeft;		// wann der Spieler das Spiel verlassen hat
	int				rank;			// position des spielers
	int				handicap;
	int				charschatted;	// wieviele Zeichen gesprochen wurden
	int				wins, losses;	// in tourney mode
	bool			used;			// ob der clientslot frei ist
	int				powerups;		// so can display quad/flag status
	bool			DataChanged;	// = 1 wenn configsting updaten kam

	playerState_t	playerState;	// die dynamischen spilerdaten


	// when clientinfo is changed, the loading of models/skins/sounds
	// can be deferred until you are dead, to prevent hitches in
	// gameplay
	char			modelName[MAX_QPATH];
	char			skinName[MAX_QPATH];
} clientInfo_t;

typedef struct snapshot_s {
	int				seqNr;				// die seqeunc number des snapshots
	// int				snapFlags;			// SNAPFLAG_RATE_DELAYED, etc
	// int				ping;

	int				serverTime;		// server time the message is valid for (in msec)

	// byte			areamask[MAX_MAP_AREA_BYTES];		// portalarea visibility bits

	playerState_t	ps;						// complete information about the current player at this time

	int				numEntities;			// all of the entities that need to be presented
	int				ringbuff_pos;			// ab dieser Position sind im Ringbuffer die numEntities viele Entities des Snapshots
											// ersetzt entities[Max_entities_in snapshot]

	int				numServerCommands;		// text based server commands to execute when this
	int				serverCommandSequence;	// snapshot becomes current
} snapshot_t;

typedef struct cgs_s
{
	int				ArgusPlayerNum;			// die spieler nummer von Argus, damit er sich selbst erkennt
	clientInfo_t	clientinfo[MAX_CLIENTS];	// die spieler client
	int				clientnum;				// anzahl der spieler
	int				localclientnum;			// welcher Client bin ich ??
	bool			ConfigStringUpdate;		// falls sich map, splier namen oder sonstige gameeinstellungen gendert haben
	bool			PlayerInfoUpdate;		// falls sich spielerdaten gendert haben 
	bool			receivedGameState;		// true if gamestate is received

	/// orginal Quake vars
	
	int				serverCommandSequence;	// aktuelle servercommand nr
	int				clientCommandSequence;	// zuletzt besttigter clinet command
	
	int				processedSnapshotNum;// the number of snapshots cgame has requested

	int				latestSnapshotNum;	// the number of snapshots the client system has received
	int				lastSnapshotTime;	// last server time
	
	snapshot_t *	snap;				// der aktuelle snapshot
	snapshot_t *	lastSnap;			// der snapshot vor dem aktuellen

	int				syncronousClients;	// hmmmm
	char			sv_paks[MAX_INFO_STRING];		// checksummen der packs
	char			sv_referencedPaks[MAX_INFO_STRING];		// checksummen der bentigten packs
	bool			sv_pure;			// ob der server "Pure" luft
	DWORD			sv_serverid;		// wird bei den player cmd zurckgeschickt
	float			timescale;			// zeitverhltnis
	bool			sv_cheats;			// cheats erlaubt ?
	
	int				TeamScores[2];		// die punkte der teams

	int				needpass;
	char			gamename;			//baseq3
	int				bot_minplayers;		//0
	int				sv_privateClients;
	int				protocol;
	char			version[MAX_INFO_STRING];		//Q3 1.15c win-x86 Jan 13 2000
	int				sv_maxPing;
	int				sv_minPing;
	int				sv_floodProtect;
	int				sv_maxRate;
	int				warmup;						// serverzeit des spielstarts
	int				intermission;			// =1 wenn spielbergang
	char			sv_hostname[MAX_INFO_STRING];	//LOKi's Place
	char			music[MAX_INFO_STRING];		// configstring 2
	char			message[MAX_INFO_STRING];	// configstring 3
	char			motd[MAX_INFO_STRING];		// configstring 4
	
	// parsed from serverinfo
	int				gametype;
	int				dmflags;
	int				teamflags;
	int				fraglimit;
	int				capturelimit;
	int				timelimit;
	int				maxclients;
	char			mapname[MAX_QPATH];

	int				voteTime;
	int				voteYes;
	int				voteNo;
	bool			voteModified;			// beep whenever changed
	char			voteString[MAX_INFO_STRING];
	char			items[MAX_INFO_STRING];	// 01 sequence mit vorhandenen items im level

	int				levelStartTime;

	int				scores1, scores2;		// from configstrings
	int				redflag, blueflag;		// flag status from configstrings
	int				ServerTime;				// die aktuelle Serverzeit

} cgs_t;

typedef struct gitem_s {
	char		*classname;		// spawning name "ammo_bullets"
	HICON		icon;			// "icons/icona_machinegun",
	char		*pickup_name;	// for printing on pickup "Bullets",
	int			quantity;		// for ammo how much, or duration of powerup 		50
	int			giType;			// IT_* flags IT_AMMO
	int			giTag;			// WP_MACHINEGUN,
} gitem_t;

//======================================================================


// all cg.stepTime, cg.duckTime, cg.landTime, etc are set to cg.time when the action
// occurs, and they will have visible effects for #define STEP_TIME or whatever msec after
 
typedef struct cg_s {
	int			clientFrame;		// incremented each frame
	
	bool		demoPlayback;
	bool		loading;			// don't defer players at initial startup
	bool		intermissionStarted;	// don't play voice rewards, because game will end shortly

	// there are only one or two snapshot_t that are relevent at a time
	int			latestSnapshotNum;	// the number of snapshots the client system has received
	int			latestSnapshotTime;	// the time from latestSnapshotNum, so we don't need to read the snapshot yet

	int			snap;				// cg.snap->serverTime <= cg.time
	// int			nextSnap;			// cg.nextSnap->serverTime > cg.time, or NULL
	// int			activeSnapshots[2];

	int			frametime;		// cg.time - cg.oldTime
	int			time;			// this is the time value that the client
								// is rendering at.
	int			oldTime;		// time at last frame, used for missile trails and prediction checking

} cg_t;

// centity_t have a direct corespondence with gentity_t in the game, but
// only the entityState_t is directly communicated to the cgame
typedef struct centity_s 
{
	int				currentState;	// zeiger auf ringbuff pso
	int				currentValid;	// true if cg.frame holds this entity
	int				mapx,mapy;		// 2d coordinaten in der map 640x480
	int				previousEvent;
	int				teleportFlag;
	int				lastSeenBy;		// clinet number von wem das entity zuletzt gesehen wurde
} centity_t;

extern cgs_t			g_cgs;	// DAS globale Game objekt,das alle informationen hlt
extern centity_t		g_cg_entities[MAX_GENTITIES];		// die entities des Clients
extern entityState_t	g_ringbuff[MAX_RINGBUF_ENTITIES];	// hier werde die Entities aus den snapshots der reihe nach gespeicher
extern gitem_t			bg_itemlist[];		// liste der items
extern int				bg_numItems;		// anzahl der bekannten Items

void GameUpdate(CPacket * p);	// updated das Spiel mit den daten aus dem Pcket p
void GameInit(int serverMessageNum, int serverCommandSequence );
char *Info_ValueForKey( const char *s, const char *key );	// orginal ID routine, sorry
void CleanString(char * string);	// entfernt " \n und faben ^1
void GameUpdateSnapshot(snapshot_t * s);	// fr snapshot zu dem spieldaten hinzu
void BG_PlayerStateToEntityState( playerState_t *ps, entityState_t *s);

#endif