// statistics.h
#ifndef STATISTICS_H_INCLUDED
#define STATISTICS_H_INCLUDED
// struktur fr allgemein statistische daten
// durchschnittswert = sum / count

#include "defines.h"
#include "mathlib.h"

typedef struct statisticValue_s
{
	int		min;		// kleinst gemessener Wert
	int		max;		// grsst gemessener Wert
	int		count;		// anzahl der gemessenen Werte
	int		sum;		// summe alle gemessenen Wert
	int		current;	// zuletzt gemessener Wert
} statisticValue_t;

typedef struct playerStatistic_s
{
	statisticValue_t	ping;			// der ping
	statisticValue_t	health;			// die gesundheit
	statisticValue_t	armor;			// die rstung
	statisticValue_t	score;			// frags, kann von den kills der grossen tabelle abweichen, da ein demo auch mitten im spiel beginnen kann
	statisticValue_t	rank;			// position in der Punktwertung
	statisticValue_t	movement;		// durchschnittliche bewegung pro sekunde
	int					totalDamage;	// schaden insgesamt verursacht
	int					spawns;			// anzahl der respawns
	int					rewardCount;	// anzahl aller rewards
	int					time;			// time played
	int					lastUpdate;		// time last statistics value added
	vec3_t				lastPosition;	// time last statistics value added
	float				eff;			// efficiency
} playerStatistic_t;

typedef struct gameStatistic_s
{
	playerStatistic_t	players[MAX_CLIENTS];

	// alle kills des spiels [KILLER][VICTIM][KINDOFDEATH], if killer=victim -> suicide or killed by world
	int	kills[MAX_CLIENTS][MAX_CLIENTS][MAX_MEANS_OF_DEATH];

	// teilt die map in 80x60 zonen ein und die aufenthaltzahl pro team
	int zones[80][60][2];

	int TotalFrags;
	// datenspuren fr die graphanzeige maximal 512 samples pro spur
	int	GraphPlayers[4];		// welche spieler sind ursprung der Graphspuren 0..3
	int	GraphValueSource[2];	// Welche werte spezielle angezeigt werden (ping, score,frags,...) , gilt fr ale spieler
	int	GraphTime[GRAPH_SAMPLES];			// zeitunkte der samples 
	int	GraphValues[8][GRAPH_SAMPLES];	// die 8 sample spuren fr 4 spieler
	int	GraphPos;				// aktuelle Sample postion
	int GraphUpdateInterval;	// 0 = jedes packet, sonst = anzahl der sekunen zwischen jedem update
	int GraphLastTime;			// die zeit wenn der garph das letze mal geupdatet wurde

} gameStatistic_t;

extern gameStatistic_t	g_statistics;

void	StatisticUpdateValue(statisticValue_t * sv, int newValue);
void	StatisticCreateHTML(char * filename);
void	StatisticInit();
void	StatisticGraphNewPlayer(int playerTrack, int playerNumber);
void	StatisticGraphNewValue(int playerTrack, int Value);
void	StatisticGraphUpdate();
void	StatisticPlayerEfficiency();

#endif

